unit AstaDataModule;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, AstaDrv2, AstaMethodManager, AstaParamList,
  NdbBase, NdbAsa, NdbBasDS, NdbAsaDS;

const
  NDBDataTypeMap: array[TNativeFieldTypes] of TFieldType = (
    ftUnknown, ftString, ftInteger, ftLargeInt, ftFloat, ftCurrency,
    ftDate, ftTime, ftDateTime, ftBoolean, ftBytes, ftMemo, ftBlob);

  MAXLOGSPPRMTYPES = 30;
  SPPrmTypeMap: array[0..MAXLOGSPPRMTYPES - 1] of TNativeParamDataType = (
    ntUnknown, ntInteger, ntInteger, ntFloat, ntFloat, ntFloat, ntDate,
    ntString, ntString, ntString, ntMemo, ntBinary, ntBlob, ntDateTime,
    ntTime, ntUnknown, ntUnknown, ntUnknown, ntUnknown, ntInteger,
    ntInt64, ntInteger, ntInteger, ntInt64, ntBoolean, ntUnknown,
    ntUnknown, ntFloat, ntBinary, ntUnknown);

type
  ENoSuchTableFieldCombo = class(Exception);

  TAstaNDBDataModule = class(TDataModule)
    TableQuery: TAsaDataset;
    DataBase: TAsaSession;
    Query: TAsaDataset;
    StmtGeneral: TAsaSQL;
    ASAFindKey: TAsaDataset;
    ADSFindKeyClone: TAstaDataSet;
    ABOM: TAstaBusinessObjectManager;
    MetaDataSet: TAsaDataset;
    SpNameDataSet: TAsaDataset;
    SpColumnDataSet: TAsaDataset;
    procedure ABOMActions0Action(Sender: TObject; ADataSet: TDataSet;
      ClientParams: TAstaParamList);
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  end;

var
  dm: TAstaNDBDataModule;

implementation

uses Loginunit, Main;

{$R *.DFM}

var
  FShowLogin:Boolean;

procedure TAstaNDBDataModule.ABOMActions0Action(Sender: TObject; ADataSet: TDataSet;
  ClientParams: TAstaParamList);
var
  i, ACount: integer;
  OrderBy: string;
  FieldNames: TStringList;
  FieldValues: TStringList;
begin
  FieldNames := nil;
  FieldValues := nil;
  try
    FieldNames := TStringList.Create;
    FieldValues	:= TStringList.Create;
    OrderBy := '';
    dm.ASAFindKey.Close;
    dm.ASAFindKey.SQL.Clear;
    with ClientParams do
    begin
      FieldNames.CommaText := ParamByName('FieldNames').AsString;
      FieldValues.CommaText := ParamByName('FieldValues').AsString;
      if FieldNames.Count <> FieldValues.Count then
        raise EDatabaseError.Create('Number of column names and values do not match in FindKey' );
      dm.ASAFindKey.SQL.Add('select * from ' +
              ParamByName('TableName').AsString	+ ' where ' );

      for i := 0 to FieldNames.Count - 1 do
      begin
        dm.ASAFindKey.SQL.Add(FieldNames[i] + ' >= ' + FieldValues[i]);

        OrderBy := OrderBy + FieldNames[i];

        if i <> FieldNames.Count - 1 then
        begin
          OrderBy := OrderBy + ',';
          dm.ASAFindKey.SQL.Add( 'AND' );
        end;
      end;

      dm.ASAFindKey.SQL.Add( 'order by ' + OrderBy );
      dm.ASAFindKey.Open;

      ParamByName('RowsFetched').AsInteger := dm.ASAFindKey.RecordCount;
      ParamByName('SQL').AsString := dm.ASAFindKey.SQL.Text;

      dm.ADSFindKeyClone.CloneFieldsFromDataset(dm.ASAFindKey,False,True);
      ACount := ParamByName('RowsToReturn').AsInteger;

      with dm.ASAFindKey do
      begin
        while not EOF and (ACount > 0) do
        begin
          dm.ADSFindKeyClone.Append;
          for i := 0 to dm.ASAFindKey.Fields.Count - 1 do
            dm.ADSFindKeyClone.Fields[i].Assign(Fields[i]);
          dm.ADSFindKeyClone.Post;
          Next;
          Dec(ACount);
        end;
      end;
    end;
  finally
    FreeAndNil(FieldNames);
    FreeAndNil(FieldValues);
  end;
end;

procedure TAstaNDBDataModule.DataModuleCreate(Sender: TObject);
var
  LoginForm: TFmLogin;
  EngPath: string;
  APath: array[0..MAX_PATH - 1] of char;
  AFile: PChar;
begin
  if FShowLogin then
  begin
    LoginForm := TFmLogin.Create(Self);
    if LoginForm.ShowModal = mrOK then
    begin
      case LoginForm.cbServerVersion.ItemIndex of
        0: Database.LibraryFile := 'dbl30t.dll';
        1: Database.LibraryFile := 'dbl40t.dll';
        2: Database.LibraryFile := 'dbl50t.dll';
        3: Database.LibraryFile := 'dblib6.dll';
        4: Database.LibraryFile := 'dblib7.dll';
        5: Database.LibraryFile := 'dblib8.dll';
      end;
      Database.LoginUser := LoginForm.edLoginName.Text;
      Database.LoginPassword := LoginForm.edPassword.Text;
      Database.LoginEngineName := LoginForm.edServerName.Text;
      Database.LoginDatabase := LoginForm.edDatabase.Text;
      Database.ServerParams := LoginForm.edServerParams.Text;
      Database.LibraryPath := '';           // Use the system "PATH="
      if Trim(Database.ServerParams) <> '' then
      begin
        Database.ServerType := stServer;
        EngPath := ExtractFilePath(Database.LoginDatabase);
        if (EngPath <> '') and
           (SearchPath(PChar(EngPath), PChar(Database.LibraryFile), nil, MAX_PATH, @APath, AFile) <> 0) then
          Database.LibraryPath := EngPath;  // Use database folder as path to library
      end else
        Database.ServerType := stClient;
      Database.ClientParams := LoginForm.edConnParams.Text;
      Database.Connected := True;
      FShowLogin := False;
      LoginForm.free;
    end;
  end;
end;

procedure TAstaNDBDataModule.DataModuleDestroy(Sender: TObject);
begin
//  FMMain.AddToLog('Disposing of ' + Name,false);
end;

initialization
  FShowLogin:=True;
end.
