unit Main;
{*********************************************************}
{*     Copyright (c) 1997-98 Asta Technology Group LLC   *}
{*                 All rights reserved.                  *}
{*                 www.astatech.com                      *}
{*********************************************************}

{$D+,L+}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ScktComp, AstaServer, StdCtrls, Db, Grids, DBGrids, ComCtrls, AstaDrv2,
  Buttons, ExtCtrls, DBCtrls, AstaDBTools, checklst, astaparamlist,
  AstaCustomSocket, AstaClientSocket, AstaStringLine, AstaMethodManager, AstaDBTypes,
  NdbAsaDS, NdbBase, NdbAsa, AstaSQLGenerate;

type
  TFmMain = class(TForm)
    AstaServerSocket1: TAstaServerSocket;
    PageControl: TPageControl;
    MemoTabsheet: TTabSheet;
    mRequests: TMemo;
    ConnectedTabSheet: TTabSheet;
    UserDataSet: TAstaDataset;
    DataSource1: TDataSource;
    UserGrid: TDBGrid;
    QueryDS: TDataSource;
    TabSheet3: TTabSheet;
    ErrorMemo: TMemo;
    DBMSDataSet: TAstaDataSet;
    ServerDataSets: TAstaDataSet;
    UserDataSetAddress: TStringField;
    UserDataSetActivity: TStringField;
    UserDataSetDate: TDateTimeField;
    TableDataSet: TAstaDataSet;
    ColumnDataSet: TAstaDataSet;
    Panel1: TPanel;
    CBEnableLogging: TCheckBox;
    AstaSPNameDataSet: TAstaDataSet;
    AstaStoredProcColumnDataSet: TAstaDataSet;
    AstaSQLGenerator1: TAstaSQLGenerator;
    procedure FormCreate(Sender: TObject);
    procedure AstaServerSocket1ClientDisconnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure AstaServerSocket1ClientConnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure AstaServerSocket1SubmitSQL(Sender: TObject;
      ClientSocket: TCustomWinSocket; AQuery: TDataSet; DataBaseStr,
      SQLString: String; Options: TAstaSelectSQLOptionSet);
    procedure AstaServerSocket1ThreadedDBSupplySession(Sender: TObject;
      ClientSocket: TCustomWinSocket; DataBaseString: String;
      var ASession: TComponent);
    procedure AstaServerSocket1FetchMetaData(Sender: TObject;
      ClientSocket: TCustomWinSocket; MetaRequest: TAstaMetaData;
      DataBaseStr, Arg1, Arg2: String);
    procedure AstaServerSocket1ShowServerMessage(Sender: TObject;
      Msg: String);
    procedure AstaServerSocket1ClientLogin(Sender: TObject; UserName,
      Password, AppName: String; var Verified: Boolean);
    procedure AstaServerSocket1ThreadedDBSupplyQuery(Sender: TObject;
      ClientSocket: TCustomWinSocket; DBAction: TThreadDbAction;
      DataBaseStr: String; CreateNew: Boolean; var AQuery: TComponent;
      SQLOptions: TAstaSelectSQLOptionSet);
    procedure AstaServerSocket1ExecSQLParamList(Sender: TObject;
      ClientSocket: TCustomWinSocket; AQuery: TComponent; DataBaseStr,
      SQLString: String; var TheResult: Boolean; var Msg: String;
      ParamList: TAstaParamList);
    procedure AstaServerSocket1TransactionBegin(Sender: TObject;
      ClientSocket: TCustomWinSocket; Session: TComponent;
      TransActionName: String);
    procedure AstaServerSocket1TransactionEnd(Sender: TObject;
      ClientSocket: TCustomWinSocket; Session: TComponent;
      Success: Boolean);
    procedure AstaServerSocket1ThreadedDBSupplyServerDataSet(
      Sender: TObject; ClientSocket: TCustomWinSocket; Session,
      OriginalServerDataSet: TComponent; var AQuery: TComponent);
    procedure AstaServerSocket1InsertAutoIncrementFetch(Sender: TObject;
      ClientSocket: TCustomWinSocket; AQuery: TComponent; TheDatabase,
      TheTable, TheAutoIncField: String; var AutoIncrementValue: Integer);
    procedure AstaServerSocket1ClientError(Sender: TObject;
      Socket: TCustomWinSocket; ErrorEvent: TErrorEvent;
      var ErrorCode: Integer);
    procedure AstaServerSocket1FetchBlobEvent(Sender: TObject;
      ClientSocket: TCustomWinSocket; AQuery: TComponent; DataBaseStr,
      TableName, FieldName, WhereString: String; var M: TMemoryStream);
    procedure CBEnableLoggingClick(Sender: TObject);
    procedure ABOMActions1Action(Sender: TObject; ADataSet: TDataSet;
      ClientParams: TAstaParamList);
    procedure AstaServerSocket1ClientDBLogin(Sender: TObject; TheUserName,
      ThePassword, TheAppName: String; var Verified: Boolean;
      var ADataBase: TComponent);
  private
    FLogging: Boolean;
    procedure UpdateUserDataSet(Address: string; Connected: Boolean);
  public
    FExceptionMessage: string;
    Counter: LongInt;
    FUserName, FPassword: string;
    procedure AddToLog(Value: string; OverrideLog: Boolean);
    procedure LogException(Socket:TCustomWinSocket;ErrorMsg: string; SendToClient: Boolean);
    procedure SetupDBMSDataBase;
    procedure AstaException(Sender: TObject; E: Exception);
    procedure PopulateTableNames;
    procedure PopulateViewNames;
    procedure PopulateStoredProcNames;
    procedure PopulateStoredProcColumns(SPName: String);
    procedure UpdateStoredProcColumnInfo;
    procedure PopulateFieldNames(TableName:String);
    procedure PopulatePrimeKeys(TableName:String);
    procedure PopulateIndexes(TableName:String);
    procedure PopulateForeignKeys(TableName: String);
  end;

var
  FmMain: TFmMain;

implementation

{$R *.DFM}

uses AstaAbout, AstaUtil, AstaClientDataset, AstaThread, AstaDataModule, LoginUnit;

const
  AstaDataBase = 'AstaDataBase';
  OVERRIDE_LOG_OPTION = True;
  NO_OVERRIDE_LOG_OPTION = False;

procedure TFmMain.AstaException(Sender: TObject; E: Exception);
var
  Sendit: Boolean;
begin
  Sendit := True;
  if E is ESocketError then
  begin
    //Sendit := False;
    FExceptionMessage := ESocketError(ExceptObject).Message;
    errormemo.lines.add(FExceptionMessage);
    Exit;
  end else
    if E is EDataBaseError then
    begin
      FExceptionMessage := EDataBaseError(E).Message
    end else
      FExceptionMessage := E.Message;
  LogException(Sender as tCustomWinSocket,FExceptionMessage, Sendit);
end;

procedure TFmMain.LogException(Socket:TCustomWinSocket;ErrorMsg: string; SendToClient: Boolean);
begin
  if SendToClient then
    AstaServerSocket1.SendExceptionToClient(Socket,ErrorMsg);
  AddToLog( ErrorMsg, OVERRIDE_LOG_OPTION );
end;

procedure TFmMain.SetupDBMSDataBase;
begin
  with DBMSDataSet do
    AppendRecord([AstaServerSocket1.AstaServerName,
      AstaServerSocket1.DataBaseName, '',ExtractFileName(Application.EXEName)]);
  UpdateAstaAlias(DBMSDataSet);
end;

procedure TFmMain.FormCreate(Sender: TObject);
begin
  FExceptionMessage := '';
  if not dm.Database.Connected then
    Halt;

  AstaServerSocket1.RegisterServerMethod('AstaFindKey','dm.FindKey');
  AstaServersocket1.RegisterDataModule(Self);
  AstaServersocket1.RegisterDataModule(dm);
  Application.OnException := AstaException;

  //this is important to get SQL work bench to Function!  Do not remove
  AstaServerSocket1.CommandLinePortcheck;
  AstaServerSocket1.Active := True;
  Caption := Caption + ' ' + AstaVersion + ' listening on port ' + IntToStr(AstaServerSocket1.Port);

  SetupDBMSDataBase;
  Counter := 0;
  PopulateTableNames;
  AstaServerSocket1.CreateSessionsForDbThreads(False);

  case AstaServerSocket1.UseThreads of
    tmSingleSession: Caption := Caption;
    tmPooledSessions: Caption := Caption + ' ' + IntToStr(AstaServerSocket1.SessionList.Count) + ' Sessions ';
    tmPersistentSessions: Caption := Caption + ' Persistent Threaded Sessions';
  end;

  Caption := Caption + ' Serving from ' + GetThePcsIPAddress;
  FLogging := True;
end;

procedure TFmMain.PopulateTableNames;
begin
  with dm do
  begin
    TableQuery.Close;
    TableQuery.Open;
    TableDataSet.DisableControls;
    TableDataSet.Empty;
    while not TableQuery.Eof do
    begin
      TableDataSet.AppendRecord([Tablequery.FieldbyName('table_name').AsString]);
      TableQuery.Next;
    end;
    TableDataSet.First;
    TableDataSet.EnableControls;
    AstaServerSocket1.MetaDataSet := TableDataSet;
  end;
end;

procedure TFmMain.PopulateViewNames;
begin
  with dm do
  begin
    MetaDataSet.Close;
    MetaDataSet.Close;
    MetaDataSet.SQL.Clear;
    MetaDataSet.SQL.Add('select viewname from sysviews');
    MetaDataSet.Open;
    TableDataSet.DisableControls;
    TableDataSet.Empty;
    while not MetaDataSet.Eof do
    begin
      TableDataSet.AppendRecord([MetaDataSet.FieldbyName('viewname').AsString]);
      MetaDataSet.Next;
    end;
    TableDataSet.First;
    TableDataSet.EnableControls;
    AstaServerSocket1.MetaDataSet := TableDataSet;
  end;
end;

procedure TFmMain.PopulateStoredProcNames;
begin
  with dm do
  begin
    SpNameDataSet.Close;
    SpNameDataSet.SQL.Clear;
    SpNameDataSet.SQL.Add('select left(string("name"+''.''+proc_name),250) as proc_name ' +
                          'from sys.sysprocedure, dbo.sysusers ' +
                          'where creator>0 and creator=dbo.sysusers.uid');
    SpNameDataSet.Open;
    AstaSPNameDataSet.Empty;
    while not SpnameDataSet.Eof do
    begin
      AstaSpNameDataSet.AppendRecord([SpNameDataSet.FieldbyName('proc_name').AsString]);
      SpnameDataSet.Next;
    end;
    AstaServerSocket1.MetaDataSet:=AstaSpNameDataSet;
  end;
end;

procedure TFmMain.PopulateStoredProcColumns(SPName: String);
begin
  with dm do
  begin
    if pos('.',SPName)>0 then
      SPName:=StringAfterToken(SPName,'.');
    SpColumnDataSet.Close;
    SpColumnDataSet.SQL.Clear;
    SpColumnDataSet.SQL.Add('select parm_name, ' +
      '(if (parm_mode_in=''Y'' and parm_mode_out=''N'' and parm_type=0) then 1 else ' +
      '(if (parm_mode_in=''Y'' and parm_mode_out=''Y'' and parm_type=0) then 2 else ' +
      '(if (parm_mode_in=''N'' and parm_mode_out=''Y'' and parm_type=0) then 3 else ' +
      '(if (parm_mode_in=''N'' and parm_mode_out=''Y'' and parm_type=4) then 4 else 0 ' +
      'endif) endif) endif) endif) as parm_type, ' +
      'domain_id as data_type ' +
      'from sys.sysprocparm join sys.sysprocedure ' +
      'where proc_name=:proc_name ' +
      'order by parm_id');
    SpColumnDataSet.ParamByName('proc_name').AsString := SPName;
    AddToLog(SpColumnDataSet.SQL.Text, NO_OVERRIDE_LOG_OPTION);
    SpColumnDataSet.Open;
    UpdateStoredProcColumnInfo;
    AstaServerSocket1.MetaDataSet := AstaStoredProcColumnDataSet;
  end;
end;

procedure TFmMain.UpdateStoredProcColumnInfo;
var
  ftype, inout: Integer;
  DType: TNativeFieldTypes;
begin
  AstaStoredProcColumnDataSet.Open;
  AstaStoredProcColumnDataSet.Empty;
  with dm.SpColumnDataSet do
  begin
    AddToLog(IntToStr(RecordCount) + ' params', NO_OVERRIDE_LOG_OPTION);
    First;
    while not EOF do
    begin
      AstaStoredProcColumnDataSet.Append;
      AstaStoredProcColumnDataSet.FieldByName('Name').AsString := FieldByName('parm_name').AsString;
      inout := FieldByName('parm_type').AsInteger;
      case inout of
        1: AstaStoredProcColumnDataSet.FieldbyName('InputType').AsInteger := ord(ptInput);
        2: AstaStoredProcColumnDataSet.FieldbyName('InputType').AsInteger := ord(ptInputOutput);
        3: AstaStoredProcColumnDataSet.FieldbyName('InputType').AsInteger := ord(ptOutput);
        4: AstaStoredProcColumnDataSet.FieldbyName('InputType').AsInteger := ord(ptResult);
      else
        AstaStoredProcColumnDataSet.FieldbyName('InputType').AsInteger := ord(ptUnknown);
      end;
      ftype := FieldByName('data_type').AsInteger;
      if ftype < MAXLOGSPPRMTYPES then
        DType := SPPrmTypeMap[ftype]
      else
        DType := ntUnknown;
      AstaStoredProcColumnDataSet.FieldbyName('DataType').AsInteger :=  ord(NDBDataTypeMap[DType]);
      AstaStoredProcColumnDataSet.Post;
      Next;
    end;
  end;
end;

procedure TFmMain.PopulateFieldNames(TableName: String);
begin
  with dm do
  begin
    if pos('.',TableName)>0 then
      TableName:=StringAfterToken(TableName,'.');
    MetaDataSet.Close;
    MetaDataSet.SQL.Clear;
    MetaDataSet.SQL.Add('select sc.column_name, sd.domain_name as fieldtype, ' +
                        'sc.width, sc.scale from systable st, syscolumn sc, sysdomain sd ' +
                        'where st.table_name=:tablename and ' +
                        'st.table_id=sc.table_id and ' +
                        'sc.domain_id=sd.domain_id ' +
                        'order by sc.column_id');
    MetaDataSet.ParamByName('tablename').AsString := TableName;
    MetaDataSet.Open;
    AstaServerSocket1.MetaDataSet := MetaDataSet;
  end;
end;

procedure TFmMain.PopulatePrimeKeys(TableName:String);
begin
  with dm do
  begin
    if pos('.',TableName)>0 then
      TableName:=StringAfterToken(TableName,'.');
    MetaDataSet.Close;
    MetaDataSet.SQL.Clear;
    MetaDataSet.SQL.Add('select sc.column_name from systable st, syscolumn sc ' +
                        'where st.table_name=:tablename and ' +
                        'st.table_id=sc.table_id and sc.pkey=''Y'' ' +
                        'order by sc.column_id');
    MetaDataSet.ParamByName('tablename').AsString := TableName;
    MetaDataSet.Open;
    AstaServerSocket1.MetaDataSet := MetaDataSet;
  end;
end;

procedure TFmMain.PopulateIndexes(TableName:String);
begin
  with dm do
  begin
    if pos('.',TableName)>0 then
      TableName:=StringAfterToken(TableName,'.');
    MetaDataSet.Close;
    MetaDataSet.SQL.Clear;
    MetaDataSet.SQL.Add('select si.index_name from systable st, sysindex si ' +
                        'where st.table_name=:tablename and ' +
                        'st.table_id=si.table_id ' +
                        'order by si.index_name');
    MetaDataSet.ParamByName('tablename').AsString := TableName;
    MetaDataSet.Open;
    AstaServerSocket1.MetaDataSet := MetaDataSet;
  end;
end;

procedure TFmMain.PopulateForeignKeys(TableName: String);
begin
  with dm do
  begin
    if pos('.',TableName)>0 then
      TableName:=StringAfterToken(TableName,'.');
    MetaDataSet.Close;
    MetaDataSet.SQL.Clear;
    MetaDataSet.SQL.Add('select sf.role from systable st, sysforeignkey sf ' +
                        'where st.table_name=:tablename and ' +
                        'st.table_id=sf.foreign_table_id ' +
                        'order by sf.foreign_key_id');
    MetaDataSet.ParamByName('tablename').AsString := TableName;
    MetaDataSet.Open;
    AstaServerSocket1.MetaDataSet := MetaDataSet;
  end;
end;

procedure TFmMain.UpdateUserDataSet(Address: string; Connected: Boolean);
begin
  with UserDataSet do
  begin
    Append;
    FieldbyName('Address').AsString := Address;
    if Connected then
      FieldbyName('Activity').AsString := 'Connected'
    else
      FieldbyName('Activity').AsString := 'Disconnected';
    FieldByName('Date').AsDATeTime := Now;
    Post;
  end;
end;

procedure TFmMain.AstaServerSocket1ClientDisconnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  UpdateUserDataSet(Socket.RemoteAddress, False);
end;

procedure TFmMain.AstaServerSocket1ClientConnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  UpdateUserDataSet(Socket.RemoteAddress, True);
end;

procedure TFmMain.AstaServerSocket1SubmitSQL(Sender: TObject;
  ClientSocket: TCustomWinSocket; AQuery: TDataSet; DataBaseStr,
  SQLString: String; Options: TAstaSelectSQLOptionSet);
begin
  AQuery.Close;
  //the above is only for single threaded model
  with AQuery as TAsaDataSet do
  begin
    SQL.Clear;
    SQL.Add(SQLString);
    AddToLog( Session.Name + ' ' + SQLString, NO_OVERRIDE_LOG_OPTION );
    //so we can show that it can run threaded
    Open;
 end;
end;

procedure TFmMain.AstaServerSocket1ThreadedDBSupplySession(Sender: TObject;
  ClientSocket: TCustomWinSocket; DataBaseString: String;
  var ASession: TComponent);
begin
  try
    ASession := TAstaNDBDataModule.Create(nil);
    with TAstaNDBDataModule(ASession).Database do
    begin
      LibraryFile := dm.Database.LibraryFile;
      LibraryPath := dm.Database.LibraryPath;
      LoginUser := dm.Database.LoginUser;
      LoginPassword := dm.Database.LoginPassword;
      LoginEngineName := dm.Database.LoginEngineName;
      LoginDatabase := dm.Database.LoginDatabase;
      ServerParams := dm.Database.ServerParams;
      ServerType := dm.Database.ServerType;
      ClientParams := dm.Database.ClientParams;
      Connected := True;
      AstaServerSocket1.SendCodedMessage(ClientSocket, 1000, Asession.Name + ' created ');
      AstaServerSocket1.recordServerActivity(Clientsocket, 'Session Created ' + Asession.Name);
    end;
  except
    LogException(ClientSocket ,'DBSupplySession ' + EDataBaseError(ExceptObject).Message, True);
  end;
end;

procedure TFmMain.AstaServerSocket1FetchMetaData(Sender: TObject;
  ClientSocket: TCustomWinSocket; MetaRequest: TAstaMetaData; DataBaseStr,
  Arg1, Arg2: String);
begin
  try
    case MetaRequest of
      mdFields:
        begin
          AddToLog('Fields Info for ' + Arg1, NO_OVERRIDE_LOG_OPTION);
          if Arg1 = '' then
           Exit;
          PopulateFieldNames(Arg1);
        end;
      mdIndexes:
        begin
          AddToLog('Indexes for ' + Arg1, NO_OVERRIDE_LOG_OPTION);
          PopulateIndexes(Arg1);
        end;
      mdPrimeKeys:
        begin
          AddToLog('Primary keys for ' + Arg1, NO_OVERRIDE_LOG_OPTION);
          PopulatePrimeKeys(Arg1);
        end;
      mdForeignKeys:
        begin
          AddToLog('Foreign keys for ' + Arg1, NO_OVERRIDE_LOG_OPTION);
          PopulateForeignKeys(Arg1);
        end;
      mdStoredProcColumns:
        begin
          AddToLog('Stored Proc Param Info for ' + Arg1, NO_OVERRIDE_LOG_OPTION);
          PopulateStoredProcColumns(Arg1);
        end;
      mdStoredProcs:
        begin
          AddToLog('Stored Procedure List' + Arg1, NO_OVERRIDE_LOG_OPTION);
          PopulateStoredProcNames;
        end;
      mdDBMSName:
        begin
          AddToLog('DBMS Info Request', NO_OVERRIDE_LOG_OPTION);
          AstaServerSocket1.MetaDataSet := DBMSDataSet;
        end;
      mdServerDataSets:
        begin
          AddToLog('Server DataSet Info Request', NO_OVERRIDE_LOG_OPTION);
          AstaServerSocket1.MetaDataSet := ServerDataSets;
        end;
      mdViews:
        begin
          AddToLog('View Info ' + Arg1, NO_OVERRIDE_LOG_OPTION);
          PopulateViewNames;
        end;
      else begin
        AddToLog('Table Names', NO_OVERRIDE_LOG_OPTION);
        PopulateTableNames;
      end;
    end;
  except
    LogException(ClientSocket,'MetaData Error: ' + EDataBaseError(ExceptObject).Message, True);
  end;
end;

procedure TFmMain.AstaServerSocket1ShowServerMessage(Sender: TObject;
  Msg: String);
begin
  AddToLog(Msg, NO_OVERRIDE_LOG_OPTION);
end;

procedure TFmMain.AstaServerSocket1ClientLogin(Sender: TObject; UserName,
  Password, AppName: String; var Verified: Boolean);
begin
  Verified := True;
end;

procedure TFmMain.AstaServerSocket1ThreadedDBSupplyQuery(Sender: TObject;
  ClientSocket: TCustomWinSocket; DBAction: TThreadDbAction;
  DataBaseStr: String; CreateNew: Boolean; var AQuery: TComponent;
  SQLOptions: TAstaSelectSQLOptionSet);
begin
  //if Create new it's being called from within a thread
  try
    if Sender is TAstaThread then
    begin
      if soPackets in SQLOptions then
      begin
        AQuery := TAsaDataSet.Create(nil);
        with AQuery as TAsaDataSet do
          Session := TAstaNDBDataModule(TAstaThread(Sender).Session).DataBase;
        Exit;
      end;
      case dbaction of
        ttTransactionStart,tttransaction:
          AQuery := TAstaNDBDataModule(TAstaThread(Sender).Session).Database;
        ttexec:
          AQuery := TAstaNDBDataModule(TAstaThread(Sender).Session).StmtGeneral;
      else
        AQuery := TAstaNDBDataModule(TAstaThread(Sender).Session).Query;
      end;
      Exit;
    end else
    begin
      case dbaction of
        ttTransactionStart,ttTransaction: AQuery := dm.DataBase;
        ttexec: AQuery := dm.StmtGeneral;
      else
        AQuery := dm.Query;
      end;
    end;
  except
    LogException(ClientSocket ,'DBSupplyQuery ' + EDataBaseError(ExceptObject).Message, True);
  end;
end;

procedure TFmMain.AstaServerSocket1ExecSQLParamList(Sender: TObject;
  ClientSocket: TCustomWinSocket; AQuery: TComponent; DataBaseStr,
  SQLString: String; var TheResult: Boolean; var Msg: String;
  ParamList: TAstaParamList);
var
  i: Integer;
  ParamArray: Array of Variant;
begin
  TheResult := False;
  try
    if FLogging then
    begin
      AddToLog( 'Exec ParamList Call: ' + SQLString, NO_OVERRIDE_LOG_OPTION );
      for i := 0 to ParamList.Count - 1 do
        if ParamList[i].datatype <> ftblob then
          AddToLog(' Param ' + IntToStr(i) + '->' + ParamList[i].Name + ' ' +
            ParamList[i].AsString, NO_OVERRIDE_LOG_OPTION );
    end;
    SetLength(ParamArray, ParamList.Count);
    for i := 0 to ParamList.Count - 1 do
      ParamArray[i] := ParamList[i].Value;
    try
      if not TAsaSQL(AQuery).Execute(SQLString, ParamArray) then
        raise EDatabaseError.Create(TAsaSQL(AQuery).LastError);
    finally
      SetLength(ParamArray,0);
    end;
    TheResult := True;
  except
    LogException(ClientSocket, Exception(exceptObject).Message, True);
  end;
end;

procedure TFmMain.AstaServerSocket1TransactionBegin(Sender: TObject;
  ClientSocket: TCustomWinSocket; Session: TComponent;
  TransActionName: String);
begin
  try
    with Session as TASASession do
      StartTransaction;
    AddToLog( 'Start Transaction ' + TransactionName, NO_OVERRIDE_LOG_OPTION );
  except
    LogException(ClientSocket, Exception(exceptObject).Message, True);
  end;
end;

procedure TFmMain.AstaServerSocket1TransactionEnd(Sender: TObject;
  ClientSocket: TCustomWinSocket; Session: TComponent; Success: Boolean);
begin
  try
    with Session as TAsaSession do
      if Success then
      begin
        Commit;
        AddToLog('Transaction Committed', NO_OVERRIDE_LOG_OPTION);
      end else
      begin
        RollBack;
        AddToLog('Transaction Rolled Back', NO_OVERRIDE_LOG_OPTION);
      end;
  except
    LogException(ClientSocket, Exception(exceptObject).Message, True);
  end;
end;

procedure TFmMain.AstaServerSocket1ThreadedDBSupplyServerDataSet(
  Sender: TObject; ClientSocket: TCustomWinSocket; Session,
  OriginalServerDataSet: TComponent; var AQuery: TComponent);
begin
{}
end;

procedure TFmMain.AstaServerSocket1InsertAutoIncrementFetch(Sender: TObject;
  ClientSocket: TCustomWinSocket; AQuery: TComponent; TheDatabase,
  TheTable, TheAutoIncField: String; var AutoIncrementValue: Integer);
begin
  try
    if AQuery is TDataset then
    begin
      with AQuery as TAsaDataset do
      begin
        SQL.Clear;                      
        SQL.Add('select @@identity');
        Open;
        if not Eof then AutoIncrementValue := Fields[0].AsInteger;
      end;
    end else
    begin
      if TAsaSQL(AQuery).OpenRead('select @@identity as ainc', []) then
        AutoIncrementValue := TAsaSQL(AQuery)['ainc'];
    end;    
  except
    LogException(ClientSocket, Exception(exceptObject).Message, True);
  end;
end;

procedure TFmMain.AstaServerSocket1ClientError(Sender: TObject;
  Socket: TCustomWinSocket; ErrorEvent: TErrorEvent;
  var ErrorCode: Integer);
begin
// AstaServerSocket1.VerifyClientconnection(Socket);
end;

procedure TFmMain.AstaServerSocket1FetchBlobEvent(Sender: TObject;
  ClientSocket: TCustomWinSocket; AQuery: TComponent; DataBaseStr,
  TableName, FieldName, WhereString: String; var M: TMemoryStream);
var
  S: string;
begin
  TAsaDataSet(AQuery).Close;
  TAsaDataSet(AQuery).SQL.Clear;
  S := 'Select ' + TableName + '.' + FieldName + ' from ' + tablename + ' ' + WhereString;
  AddToLog('Blob Request ' + S, NO_OVERRIDE_LOG_OPTION);
  S := StripCRLF(S);
  TAsaDataSet(AQuery).SQL.Add(S);
  TAsaDataSet(AQuery).Open;
  if TAsaDataSet(AQuery).Recordcount > 0 then
  begin
    case TAsaDataSet(AQuery).FieldByName(FieldName).DataType of
      ftmemo: TMemoField(TAsaDataSet(AQuery).FieldByName(FieldName)).SaveToStream(M);
      ftblob, ftGraphic: TBlobField(TAsaDataSet(AQuery).FieldbyName(FieldName)).SaveToStream(M);
    end;
  end;
end;

procedure TFmMain.AddToLog(Value: string; OverrideLog: Boolean);
begin
  if FLogging or OverrideLog then
    mrequests.lines.Add(Value);
end;

procedure TFmMain.CBEnableLoggingClick(Sender: TObject);
begin
  FLogging := CBEnableLogging.Checked;
  if FLogging then
    AddToLog( 'Logging enabled', OVERRIDE_LOG_OPTION )
  else
    AddToLog( 'Logging disabled', OVERRIDE_LOG_OPTION );
end;

procedure TFmMain.ABOMActions1Action(Sender: TObject; ADataSet: TDataSet;
  ClientParams: TAstaParamList);
var
  S: string;
begin
  with ClientParams do
  begin
    S := ParamByName('VarIn').AsString;
    ParamByName('StrOut').AsString := S;
  end;
end;

procedure TFmMain.AstaServerSocket1ClientDBLogin(Sender: TObject;
  TheUserName, ThePassword, TheAppName: String; var Verified: Boolean;
  var ADataBase: TComponent);
begin
  ADataBase := TAstaNDBDataModule.Create(nil);
  with TAstaNDBDataModule(ADataBase).Database do
  begin
    if instring('Exception',TheAppName) then raise Exception.Create('ONClienDBLogin simulated Exception' +TheAppName);
    LibraryFile := dm.Database.LibraryFile;
    LibraryPath := dm.Database.LibraryPath;
    LoginUser := dm.Database.LoginUser;
    LoginPassword := dm.Database.LoginPassword;
    LoginEngineName := dm.Database.LoginEngineName;
    LoginDatabase := dm.Database.LoginDatabase;
    ServerParams := dm.Database.ServerParams;
    ServerType := dm.Database.ServerType;
    ClientParams := dm.Database.ClientParams;
    Connected := True;
    AstaServerSocket1.SendCodedMessage(Sender as TCustomWinSocket,1000,TheUserName+' '+ADataBase.Name+' created ');
    AstaServerSocket1.recordServerActivity(Sender as TCustomWinSocket,'Session Created '+ADataBase.Name);
    Verified:=True;
  end;
end;

end.

